@@ -1,4 +1,4 @@
-#!/usr/bin/perl
+#! perl
 
 package main;
 
@@ -10,12 +10,13 @@ package ChordPro::Output::Common;
 use strict;
 use warnings;
 use ChordPro::Chords;
-use ChordPro::Utils qw( demarkup );
+use ChordPro::Utils qw( demarkup is_true );
 use String::Interpolate::Named;
 use utf8;
 use POSIX qw(setlocale LC_TIME strftime);
+use Ref::Util qw( is_arrayref );
 
-use parent qw(Exporter);
+use Exporter 'import';
 our @EXPORT;
 our @EXPORT_OK;
 
@@ -30,8 +31,9 @@ sub fmt_subst {
 	####CHECK
 	$m->{_key} =
 	  [ map { ChordPro::Chords::transpose( $_, $capo ) }
-	        @{$m->{key}} ];
+	    @{$m->{key}} ];
     }
+    $m->{capo} = [] if $config->{settings}->{decapo};
     $m->{key_actual} //= $m->{key};
     $m->{tuning} //= [ join(" ", ChordPro::Chords::get_tuning) ];
     # If config->{instrument} is missing, or null, the program abends with
@@ -48,15 +50,17 @@ sub fmt_subst {
 	$m->{"user.fullname"} = [ $config->{user}->{fullname} ];
     }
     setlocale( LC_TIME, "" );
-    $m->{today} //= [ strftime( $config->{dates}->{today}->{format},
-				localtime(time) ) ];
-
+    $m->{today} //= strftime( $config->{dates}->{today}->{format},
+			      localtime(time) );
+    $m->{chordpro} = "ChordPro";
+    $m->{"chordpro.version"} = $ChordPro::VERSION;
     for ( keys %{ $config->{settings} } ) {
 	my $v = $config->{settings}->{$_};
 	$v = '' if $v =~ /^(0|false|off)$/i;
 	$v = 1  if $v=~ /^(true|on)$/i;
 	$m->{"settings.$_"} = $v;
     }
+
     interpolate( { %$s, args => $m,
 		   separator => $config->{metadata}->{separator} },
 		 $t );
@@ -127,7 +131,7 @@ sub Roman($) {
 push( @EXPORT_OK, 'Roman' );
 
 sub roman($) {
-    lc Roman shift;
+    lc( Roman(shift) );
 }
 push( @EXPORT_OK, 'roman' );
 
@@ -139,6 +143,16 @@ push( @EXPORT_OK, 'roman' );
 #sub PODBG() { $config->{debug}->{x1} }
 sub PODBG() { 0 }
 
+# Suppress toc entry.
+sub _suppresstoc {
+    my ( $meta ) = @_;
+    return !is_true($meta->{_TOC}->[0]) if exists($meta->{_TOC});
+    # return unless exists($meta->{sorttitle});
+    # my $st = $meta->{sorttitle};
+    # defined($st) && ( $st->[0] eq "" || $st->[0] eq '""' );
+    return;
+}
+
 sub prep_outlines {
     my ( $book, $ctl ) = @_;
     return [] unless $book && @$book; # unlikely
@@ -149,13 +163,15 @@ sub prep_outlines {
     my @fields = map { /^[-+]*(.*)/ ? $1 : $_ } @{$ctl->{fields}};
     if ( @fields == 1 && $fields[0] eq "songindex" ) {
 	# Return in book order.
-	return [ map { [ $_->{meta}->{songindex}, $_ ] } @$book ];
+	return [ map { [ $_->{meta}->{songindex}, $_ ] }
+		 grep { !_suppresstoc($_->{meta}) } @$book ];
     }
     return $book unless @fields; # ?
 
     my @book;
     foreach my $song ( @$book ) {
-	my $meta = $song->{meta};
+	my $meta = { %{$song->{meta}} };
+	next if _suppresstoc($meta);
 
 	my @split;
 
@@ -164,7 +180,7 @@ sub prep_outlines {
 	    push( @split, [ $coreitem, [""] ] ), next unless $meta->{$coreitem};
 
 	    my @s = map { [ $_ ] }
-	      @{ UNIVERSAL::isa( $meta->{$coreitem}, 'ARRAY' )
+	      @{ is_arrayref( $meta->{$coreitem} )
 		? $meta->{$coreitem}
 		: [ $meta->{$coreitem} ]
 	    };
@@ -259,16 +275,20 @@ sub prep_outlines {
 
     # Sort.
     my $i = -1;
+    use Unicode::Collate;
+    my $collator = Unicode::Collate->new;
     my $srt =
       "sub { " .
       join( " or ",
 	    map { $i++;
 		  my ( $rev, $f ) = /^([-+]*)(.*)/;
 		  my $num = $rev =~ s/\+//g;
+		  my ( $a, $b ) = $rev =~ /-/ ? qw( b a ) : qw( a b );
+		  my $l = "\$$a"."->[$i]";
+		  my $r = "\$$b"."->[$i]";
 		  warn("F: $f, N: $num, R: $rev\n") if PODBG;
-		  "\$" . ( $rev =~ /-/ ? "b" : "a" ) . "->[$i] " .
-		  ($num ? '<=>' : 'cmp') .
-		  " \$" . ( $rev =~ /-/  ? "a" : "b" ) . "->[$i]" }
+		  $num ? "$l <=> $r" : "\$collator->cmp( $l, $r )"
+	       }
 		@{$ctl->{fields}} ) .
       " }";
     warn("SRT; $srt\n") if PODBG;
@@ -276,7 +296,11 @@ sub prep_outlines {
     @book =
       sort $srt
       map { my $t = $_;
-	    [ ( map { demarkup(lc($t->{meta}->{$_}->[0] // "")) }
+	    [ ( map { demarkup(lc(  ( index($_,"sort") && is_arrayref($t->{meta}->{"sort$_"})
+				      ? $t->{meta}->{"sort$_"}->[0]
+				      : undef ) //
+				   $t->{meta}->{$_}->[0] //
+				   "")) }
 		    @fields ),
 	      $_ ] }
 	  @book;
