@@ -5,18 +5,23 @@ use warnings;
 use utf8;
 use Carp;
 
+use FindBin;
+use lib "$FindBin::Bin/../lib";
+use lib "$FindBin::Bin/../lib/ChordPro/lib";
+
 binmode STDOUT => ':utf8';
 binmode STDERR => ':utf8';
 
 package ChordPro::Testing;
 
-use base 'Exporter';
+use parent 'Exporter';
 our @EXPORT = qw( $config );
 
 use Test::More ();
 
-use App::Packager ( ':name', 'ChordPro' );
+use ChordPro::Files;
 use ChordPro::Config;
+use ChordPro::Paths;
 use ChordPro::Chords;
 
 sub import {
@@ -26,7 +31,6 @@ sub import {
     -d "t" && chdir "t";
 
     $::running_under_test = 1;
-    App::Packager->export_to_level(1);
     Test::More->export_to_level(1);
     $pkg->export_to_level( 1, undef, @EXPORT );
 }
@@ -37,7 +41,7 @@ sub is_deeply {
     if ( ref($got) eq 'HASH' && ref($expect) eq 'HASH' ) {
 	fixchords($got) if $got->{body};
 
-	for ( qw( config ) ) {
+	for ( qw( config generate ) ) {
 	    delete $got->{$_} unless exists $expect->{$_};
 	}
 	if ( $got->{chordsinfo} ) {
@@ -50,7 +54,9 @@ sub is_deeply {
 		}
 	    }
 	}
-	for ( qw( instrument user key_from key_actual chords numchords ) ) {
+	for ( qw( instrument user key_from key_actual chords numchords
+		  _configversion bookmark
+	       ) ) {
 	    delete $got->{meta}->{$_} unless exists $expect->{meta}->{$_};
 	}
     }
@@ -76,7 +82,7 @@ ChordPro::Chords::add_config_chord
 no warnings 'redefine';
 
 sub getresource {
-    App::Packager::U_GetResource(@_);
+    CP->findres($_[0]);
 }
 }
 
@@ -104,15 +110,13 @@ sub cmp {
     }
 }
 
-use File::LoadLines qw( loadlines );
-
 sub differ {
     my ($file1, $file2) = @_;
     $file2 = "$file1" unless $file2;
     $file1 = "$file1";
 
-    my @lines1 = loadlines($file1);
-    my @lines2 = loadlines($file2);
+    my @lines1 = fs_load( $file1, { fail => 'hard' } );
+    my @lines2 = fs_load( $file2, { fail => 'hard' } );
     my $linesm = @lines1 > @lines2 ? @lines1 : @lines2;
     for ( my $line = 1; $line < $linesm; $line++ ) {
 	next if $lines1[$line] eq $lines2[$line];
