@@ -49,6 +49,8 @@ var user = {
 } // user
 
 var	abc				// (global for 'toxxx.js')
+var s				// (var for do_begin_end - grid3)
+var cl				// (var for grid3)
 
 abc2svg.path = []		// path to ABC files
 
@@ -75,6 +77,11 @@ function abc_cmd(cmd, args, interp_name) {
 	abc2svg.abort = function(e) {
 		abc2svg.printErr('javascript error: ' + e.message +
 			'\nStack:\n'  + e.stack)
+		if (abc) {
+			abc.parse.state = 0		// force block flush
+			abc.blk_flush()
+			abc2svg.abc_end()
+		}
 		abc2svg.quit()
 	} // abort()
 
